// Copyright 2020 Tencent Inc. All rights reserved.
//
// Author: Tencent
//
// 文件注释
#ifndef TCCC_USER_SDK_TCCC_INCLUDE_ITCCCCLOUD_H_
#define TCCC_USER_SDK_TCCC_INCLUDE_ITCCCCLOUD_H_

#include "ITCCCCallback.h"
#include "ITCCCDeviceManager.h"
#include "TCCCCode.h"
#include "TCCCTypeDef.h"

/// @defgroup ITCCCCloud_cplusplus TCCC 主功能接口
/// 腾讯云 TCCC 主功能接口
/// @{
namespace tccc {
class ITCCCCloud;
}

/// Export the following C-style interface to facilitate “LoadLibrary()”
/// You can use the following methods to create and destroy ITCCCCloud instance：
/// <pre>
///    ITCCCCloud *tcccCloud = getTCCCShareInstance();
///    if(tcccCloud) {
///        std::string version(tcccCloud->getSDKVersion());
///    }
///    //
///    //
///    destroyTCCCShareInstance();
///    tcccCloud = nullptr;
/// </pre>
///
extern "C" {
/// @name Exported C function
/// @{
#ifdef __ANDROID__
TCCC_API tccc::ITCCCCloud* getTCCCShareInstance(void* context);
#else
TCCC_API tccc::ITCCCCloud* getTCCCShareInstance();
#endif
TCCC_API void destroyTCCCShareInstance();
/// @}
}

namespace tccc {

class ITCCCCloud {
 protected:
  virtual ~ITCCCCloud(){};

 public:
  /////////////////////////////////////////////////////////////////////////////////
  //
  //                    创建实例和事件回调
  //
  /////////////////////////////////////////////////////////////////////////////////
  /// @name 创建实例和事件回调
  /// @{

  /**
   * 1.1 创建 ITCCCCloud 实例（单例模式）
   *
   * @param context 仅适用于 Android 平台，SDK 内部会将其转化为 Android 平台的 ApplicationContext 用于调用 Androud
   * System API。 如果传入的 context 参数为空，SDK 内部会自动获取当前进程的 ApplicationContext。
   * @note
   * 1. 如果您使用 delete ITCCCCloud* 会导致编译错误，请使用 destroyTCCCShareInstance 释放对象指针。
   * 2. 在 Windows、Mac 和 iOS 平台上，请调用 getTCCCShareInstance() 接口。
   * 3. 在 Android 平台上，请调用 getTCCCShareInstance(void *context) 接口。
   */
#ifdef __ANDROID__
  static tccc::ITCCCCloud* getShareInstance(void* context);
#else
  static tccc::ITCCCCloud* getShareInstance();
#endif

  /**
   * 1.2 销毁 ITCCCCloud 实例（单例模式）
   */
  static void destroyShareInstance();

  /**
   * 1.3 设置 TCCC 事件回调
   *
   * 您可以通过 {@link ITCCCCallback} 获得来自 SDK 的各类事件通知（比如：错误码，警告码，音视频状态参数等）。
   *
   * @param callback 回调
   */
  virtual void addCallback(ITCCCCallback* callback) = 0;

  /**
   * 1.4 移除 TCCC 事件回调
   *
   * @param callback 回调
   */
  virtual void removeCallback(ITCCCCallback* callback) = 0;

  /////////////////////////////////////////////////////////////////////////////////
  //
  //                    呼叫相关接口函数
  //
  /////////////////////////////////////////////////////////////////////////////////
  /// @name 呼叫相关接口函数
  /// @{

  // /**
  //  * 3.1 发起呼出
  //  *
  //  * @param param 呼出参数
  //  *
  //  * @param startCallback 呼出回调,仅仅表示发起呼叫是否成功。当onNewSession回调了表示新会话，
  //  * 对端接听用onAccepted，对端挂断用onEnded
  //  */
  // virtual void startCall(const TCCCStartCallParams& param, ITXCallback* startCallback) = 0;

  /**
   * 3.1 一键外呼
   *
   * @param param 呼出参数
   *
   * @param startCallback 呼出回调,仅仅表示发起呼叫是否成功。当onNewSession回调了表示新会话，
   * 对端接听用onAccepted，对端挂断用onEnded
   */
  virtual void startCall(const TCCCCallParams& param, ITXCallback* startCallback) = 0;

  /**
   * 3.2 结束会话
   *
   */
  virtual void terminate() = 0;

  // /**
  //  * 3.5 删除会话
  //  *
  //  * @param sessionId 会话ID
  //  *
  //  * @param callback 删除会话成功与否回调
  //  */
  // // virtual void deleteCall(const char *sessionId, ITXCallback* callback) = 0;

  // /**
  //  * 3.6 呼叫保持
  //  *
  //  * @param sessionId 会话ID
  //  *
  //  * @param callback 呼叫保持成功与否回调
  //  */
  // // virtual void hold(const char *sessionId, ITXCallback* callback) = 0;

  // /**
  //  * 3.7 取消通话保持
  //  *
  //  * @param sessionId 会话ID
  //  *
  //  * @param callback 取消通话保持成功与否回调
  //  */
  // // virtual void unHold(const char *sessionId, ITXCallback* callback) = 0;

  /**
   * 3.8 发送 DTMF（双音多频信号）
   *
   * @param digits DTMF参数是一个字符串，可以包含字符 0-9、*、#。
   *
   * @param callback 发送 DTMF成功与否回调，调用一次就会播放一次按键声音
   * @note
   *  该方法仅仅在通话建立后才生效。需要注意的是前后按键需要保证一定的间隔，否则会出现可能出现对端无法感知的问题。
   */
  virtual void sendDTMF(const char digit, ITXCallback* callback) = 0;

  // /**
  //  * 3.9 转接会话
  //  *
  //  * @param params 转接参数
  //  *
  //  * @param callback 转接会话成功与否回调
  //  */
  // virtual void transfer(TCCCtransferParams params, ITXCallback* callback) = 0;

  /// @}
  /////////////////////////////////////////////////////////////////////////////////
  //
  //                    音频相关接口函数
  //
  /////////////////////////////////////////////////////////////////////////////////
  /// @name 音频相关接口函数
  /// @{

  /**
   * 4.1 静音
   */
  virtual void mute() = 0;

  /**
   * 4.2 取消静音
   */
  virtual void unmute() = 0;

  /**
   * 5.1 开启本地音频的采集和发布
   *
   * SDK 默认不开启麦克风，当用户需要发布本地音频时，需要调用该接口开启麦克风采集，并将音频编码并发布到当前的房间中。
   * 开启本地音频的采集和发布后，房间中的其他用户会收到 {@link onUserAudioAvailable}(userId, true) 的通知。
   * @param quality 声音音质
   * - {@link
   * TCCCAudioQualitySpeech}，流畅：采样率：16k；单声道；音频裸码率：16kbps；适合语音通话为主的场景，比如在线会议，语音通话。
   * - {@link TCCCAudioQualityDefault}，默认：采样率：48k；单声道；音频裸码率：50kbps；SDK
   * 默认的音频质量，如无特殊需求推荐选择之。
   * - {@link TCCCAudioQualityMusic}，高音质：采样率：48k；双声道 +
   * 全频带；音频裸码率：128kbps；适合需要高保真传输音乐的场景，比如在线K歌、音乐直播等。
   * @note 该函数会检查麦克风的使用权限，如果当前 App 没有麦克风权限，SDK 会自动向用户申请麦克风使用权限。
   */
  virtual void startLocalAudio(TCCCAudioQuality quality) = 0;

  /**
   * 4.7 启用音量大小提示。
   *
   * 开启此功能后，SDK 会在 {@link ITCCCCallback} 中的 {@link onVoiceVolume} 回调中反馈远端音频的音量大小。
   * @note 如需打开此功能，请在 接听或者发起呼叫 之前调用才可以生效。
   * @param interval 设置 onVoiceVolume 回调的触发间隔，单位为ms，最小间隔为100ms，如果小于等于 0
   * @param enable_vad  true：打开本地人声检测 ；false：关闭本地人声检测。默认是FALSE
   则会关闭回调，建议设置为300ms；
   */
  virtual void enableAudioVolumeEvaluation(uint32_t interval, bool enable_vad) = 0;

  /**
   * 4.8 开始播放音乐
   *
   * @param path 音乐路径。
   * @param loopCount 音乐循环播放的次数。取值范围为0 - 任意正整数，默认值：1。1
   * 表示播放音乐一次,以此类推。0表示无限循环，手动停止
   */
  virtual void startPlayMusic(const char* path, int loopCount) = 0;

  /**
   * 4.9 停止播放音乐
   */
  virtual void stopPlayMusic() = 0;

  /// @}
  /////////////////////////////////////////////////////////////////////////////////
  //
  //                    视频相关接口函数
  //
  /////////////////////////////////////////////////////////////////////////////////
  /// @name 视频相关接口函数
  /// @{

/**
 * 5.2 开启本地摄像头的预览画面（桌面端）
 *
 * 在调用该接口之前，您可以先调用 setCurrentCameraDevice 选择使用 Mac 自带摄像头或外接摄像头。
 * 当开始渲染首帧摄像头画面时，您会收到 {@link TCCCCloudDelegate}  中的 onCameraDidReady 回调通知。
 * @param view 承载视频画面的控件。
 */
#if TARGET_PLATFORM_PHONE
  virtual void startLocalPreview(bool frontCamera, TCCCView view) = 0;
#endif

  /**
   * 5.2 暂停/恢复发布本地的视频流
   *
   * 该接口可以暂停（或恢复）发布本地的视频画面，暂停之后，同一房间中的其他用户将无法继续看到自己画面。
   * 该接口在指定 TCCCVideoStreamTypeBig 时等效于 start/stopLocalPreview 这两个接口，但具有更好的响应速度。
   * 因为 start/stopLocalPreview 需要打开和关闭摄像头，而打开和关闭摄像头都是硬件设备相关的操作，非常耗时。
   * 相比之下，muteLocalVideo
   * 只需要在软件层面对数据流进行暂停或者放行即可，因此效率更高，也更适合需要频繁打开关闭的场景。 当暂停/恢复发布指定
   * TCCCVideoStreamTypeBig 后，同一房间中的其他用户将会收到 onUserVideoAvailable 回调通知。 当暂停/恢复发布指定
   * TCCCVideoStreamTypeSub 后，同一房间中的其他用户将会收到 onUserSubStreamAvailable 回调通知。
   * @param streamType 要暂停/恢复的视频流类型（仅支持 {@link TCCCVideoStreamTypeBig} 和 {@link
   * TCCCVideoStreamTypeSub}）。
   * @param mute true：暂停；false：恢复。
   */
#if TARGET_PLATFORM_DESKTOP
  virtual void startLocalPreview(TCCCView view) = 0;
#endif

  /**
   * 5.3 更新本地摄像头的预览画面
   */
  virtual void updateLocalView(TCCCView view) = 0;

  /**
   * 5.4 停止摄像头预览
   */
  virtual void stopLocalPreview() = 0;

  /**
   * 5.5 暂停/恢复发布本地的视频流
   *
   * 该接口可以暂停（或恢复）发布本地的视频画面，暂停之后，同一房间中的其他用户将无法继续看到自己画面。
   * 该接口在指定 TCCCVideoStreamTypeBig 时等效于 start/stopLocalPreview 这两个接口，但具有更好的响应速度。
   * 因为 start/stopLocalPreview 需要打开和关闭摄像头，而打开和关闭摄像头都是硬件设备相关的操作，非常耗时。
   * 相比之下，muteLocalVideo
   * 只需要在软件层面对数据流进行暂停或者放行即可，因此效率更高，也更适合需要频繁打开关闭的场景。 当暂停/恢复发布指定
   * TCCCVideoStreamTypeBig 后，同一房间中的其他用户将会收到 onUserVideoAvailable 回调通知。 当暂停/恢复发布指定
   * TCCCVideoStreamTypeSub 后，同一房间中的其他用户将会收到 onUserSubStreamAvailable 回调通知。
   * @param streamType 要暂停/恢复的视频流类型（仅支持 {@link TCCCVideoStreamTypeBig} 和 {@link
   * TCCCVideoStreamTypeSub}）。
   * @param mute true：暂停；false：恢复。
   */
  virtual void muteLocalVideo(TCCCVideoStreamType streamType, bool mute) = 0;

  /**
   * 5.6 设置本地画面被暂停期间的替代图片
   *
   * 当您调用 muteLocalVideo(true)
   * 暂停本地画面时，您可以通过调用本接口设置一张替代图片，设置后，房间中的其他用户会看到这张替代图片，而不是黑屏画面。
   * @param image 设置替代图片，空值代表在 muteLocalVideo 之后不再发送视频流数据，默认值为空。
   * @param fps 设置替代图片帧率，最小值为5，最大值为10，默认5。
   */
  // virtual void setVideoMuteImage(TCCCImageBuffer* image, int fps) = 0;

  /**
   * 5.7 订阅远端用户的视频流，并绑定视频渲染控件
   *
   * 调用该接口可以让 SDK 拉取指定 userid 的视频流，并渲染到参数 view 指定的渲染控件上。您可以通过 {@link
   * setRemoteRenderParams} 设置画面的显示模式。
   * - 如果您已经知道房间中有视频流的用户的 userid，可以直接调用 startRemoteView 订阅该用户的画面。
   * - 如果您不知道房间中有哪些用户在发布视频，您可以在 enterRoom 之后等待来自 {@link onUserVideoAvailable} 的通知。
   * 调用本接口只是启动视频流的拉取，此时画面还需要加载和缓冲，当缓冲完毕后您会收到来自 {@link onFirstVideoFrame}
   * 的通知。
   * @param userId 指定远端用户的 ID。
   * @param streamType 指定要观看 userId 的视频流类型。
   *    - 高清大画面：{@link TCCCVideoStreamTypeBig}。
   *    - 低清小画面：{@link TCCCVideoStreamTypeSmall}（需要远端用户通过 {@link enableEncSmallVideoStream}
   * 开启双路编码后才有效果）。
   *    - 辅流画面（常用于屏幕分享）：{@link TCCCVideoStreamTypeSub}。
   * @param view 用于承载视频画面的渲染控件。
   * @note 注意几点规则需要您关注：
   *  1. SDK 支持同时观看某 userid 的大画面和辅路画面，或者同时观看某 userid
   * 的小画面和辅路画面，但不支持同时观看大画面和小画面。
   *  2. 只有当指定的 userid 通过 {@link enableEncSmallVideoStream} 开启双路编码后，才能观看该用户的小画面。
   *  3. 当指定的 userid 的小画面不存在时，SDK 默认切换到该用户的大画面。
   */
  virtual void startRemoteView(const char* userId, TCCCVideoStreamType streamType, TCCCView view) = 0;

  /**
   * 5.8 更新远端用户的视频渲染控件
   *
   * 该接口可用于更新远端视频画面的渲染控件，常被用于切换显示区域的交互场景中。
   * @param view 承载视频画面的控件。
   * @param streamType 要设置预览窗口的流类型（仅支持 {@link TCCCVideoStreamTypeBig} 和 {@link
   * TCCCVideoStreamTypeSub}）。
   * @param userId 指定远端用户的 ID。
   */
  virtual void updateRemoteView(const char* userId, TCCCVideoStreamType streamType, TCCCView view) = 0;

  /**
   * 5.9 停止订阅远端用户的视频流，并释放渲染控件
   *
   * 调用此接口会让 SDK 停止接收该用户的视频流，并释放该路视频流的解码和渲染资源。
   * @param userId 指定远端用户的 ID。
   * @param streamType 指定要观看 userId 的视频流类型。
   * - 高清大画面：{@link TCCCVideoStreamTypeBig}。
   * - 低清小画面：{@link TCCCVideoStreamTypeSmall}。
   * - 辅流画面（常用于屏幕分享）：{@link TCCCVideoStreamTypeSub}。
   */
  virtual void stopRemoteView(const char* userId, TCCCVideoStreamType streamType) = 0;

  /**
   * 5.10 停止订阅所有远端用户的视频流，并释放全部渲染资源
   *
   * 调用此接口会让 SDK 停止接收所有来自远端的视频流，并释放全部的解码和渲染资源。
   * @note 如果当前有正在显示的辅路画面（屏幕分享）也会一并被停止。
   */
  virtual void stopAllRemoteView() = 0;

  /**
   * 5.11 暂停/恢复订阅远端用户的视频流
   *
   * 该接口仅暂停/恢复接收指定用户的视频流，但并不释放显示资源，视频画面会被冻屏在接口调用时的最后一帧。
   * @param userId 指定远端用户的 ID。
   * @param streamType 要暂停/恢复的视频流类型（仅支持 {@link TCCCVideoStreamTypeBig} 和 {@link
   * TCCCVideoStreamTypeSub}）。
   * @param mute 是否暂停接收。
   * @note 该接口支持您在进入房间（enterRoom）前调用，暂停状态会在退出房间（exitRoom）在之后会被重置。
   */
  virtual void muteRemoteVideoStream(const char* userId, TCCCVideoStreamType streamType, bool mute) = 0;

  /**
   * 5.12 暂停/恢复订阅所有远端用户的视频流
   *
   * 该接口仅暂停/恢复接收所有用户的视频流，但并不释放显示资源，视频画面会被冻屏在接口调用时的最后一帧。
   * @param mute  是否暂停接收。
   * @note 该接口支持您在进入房间（enterRoom）前调用，暂停状态会在退出房间（exitRoom）在之后会被重置。
   */
  virtual void muteAllRemoteVideoStreams(bool mute) = 0;

  /**
   * 5.13 设置视频编码器的编码参数
   *
   * 该设置能够决定远端用户看到的画面质量，同时也能决定云端录制出的视频文件的画面质量。
   * @param param 用于设置视频编码器的相关参数，详情请参见 {@link TCCCVideoEncParam}。
   */
  virtual void setVideoEncoderParam(const TCCCVideoEncParam& param) = 0;

  /**
   * 5.15 设置本地画面的渲染参数
   *
   * 可设置的参数包括有：画面的旋转角度、填充模式以及左右镜像等。
   * @param params 画面渲染参数，详情请参见 {@link TCCCRenderParams}。
   */
  virtual void setLocalRenderParams(const TCCCRenderParams& params) = 0;

  /**
   * 5.16 设置远端画面的渲染模式
   *
   * 可设置的参数包括有：画面的旋转角度、填充模式以及左右镜像等。
   * @param userId 指定远端用户的 ID。
   * @param streamType 可以设置为主路画面（{@link TCCCVideoStreamTypeBig}）或辅路画面（{@link
   * TCCCVideoStreamTypeSub}）。
   * @param params 画面渲染参数，详情请参见 {@link TCCCRenderParams}。
   */
  virtual void setRemoteRenderParams(const char* userId, TCCCVideoStreamType streamType,
                                     const TCCCRenderParams& params) = 0;

  /**
   * 5.17 设置视频编码器输出的画面方向
   *
   * 该设置不影响本地画面的预览方向，但会影响房间中其他用户所观看到（以及云端录制文件）的画面方向。
   * 当用户将手机或 Pad 上下颠倒时，由于摄像头的采集方向没有变，所以房间中其他用户所看到的画面会变成上下颠倒的，
   * 在这种情况下，您可以通过调用该接口将 SDK
   * 编码出的画面方向旋转180度，如此一来，房间中其他用户所看到的画面可保持正常的方向。
   * 如果您希望实现上述这种友好的交互体验，我们更推荐您直接调用 {@link setGSensorMode}
   * 实现更加智能的方向适配，无需您手动调用本接口。
   * @param rotation 目前支持 0、90、180、270 两个旋转角度，默认值：TCCCVideoRotation_0，即不旋转。
   */
  virtual void setVideoEncoderRotation(TCCCVideoRotation rotation) = 0;

  /**
   * 5.18 设置编码器输出的画面镜像模式
   *
   * 该设置不影响本地画面的镜像模式，但会影响房间中其他用户所观看到（以及云端录制文件）的镜像模式。
   * @param mirror 是否开启远端镜像，true：开启远端画面镜像；false：关闭远端画面镜像，默认值：false。
   */
  virtual void setVideoEncoderMirror(bool mirror) = 0;

  /**
   * 5.20 开启大小画面双路编码模式
   *
   * 开启双路编码模式后，当前用户的编码器会同时输出【高清大画面】和【低清小画面】两路视频流（但只有一路音频流）。
   * 如此以来，房间中的其他用户就可以根据自身的网络情况或屏幕大小选择订阅【高清大画面】或是【低清小画面】。
   * @param enable 是否开启小画面编码，默认值：false。
   * @param smallVideoEncParam 小流的视频参数。
   * @return 0：成功；-1：当前大画面已被设置为较低画质，开启双路编码已无必要。
   * @note 双路编码开启后，会消耗更多的 CPU 和 网络带宽，所以 Mac、Windows 或者高性能 Pad
   * 可以考虑开启，不建议手机端开启。
   */
  virtual void enableSmallVideoStream(bool enable, const TCCCVideoEncParam& smallVideoEncParam) = 0;

  /**
   * 5.21 切换指定远端用户的大小画面
   *
   * 当房间中某个主播开启了双路编码之后，房间中其他用户通过 {@link startRemoteView} 订阅到的画面默认会是【高清大画面】。
   * 您可以通过此接口选定希望订阅的画面是大画面还是小画面，该接口在 {@link startRemoteView} 之前和之后调用均可生效。
   * @param userId 指定远端用户的 ID。
   * @param streamType 视频流类型，即选择看大画面还是小画面，默认为大画面。
   * @note 此功能需要目标用户已经通过 {@link enableEncSmallVideoStream} 提前开启了双路编码模式，否则此调用无实际效果。
   */
  virtual void setRemoteVideoStreamType(const char* userId, TCCCVideoStreamType streamType) = 0;

  /// @}

  /////////////////////////////////////////////////////////////////////////////////
  //
  //                    屏幕分享相关接口
  //
  /////////////////////////////////////////////////////////////////////////////////

  /**
   * 9.1 开始桌面端屏幕分享（该接口仅支持桌面系统）
   *
   * 该接口可以抓取整个 Mac OS 系统的屏幕内容，或抓取您指定的某个应用的窗口内容，并将其分享给同房间中的其他用户。
   * @param view 渲染控件所在的父控件，可以设置为空值，表示不显示屏幕分享的预览效果。
   * @param streamType
   * 屏幕分享使用的线路，可以设置为主路（TCCCVideoStreamTypeBig）或者辅路（TCCCVideoStreamTypeSub），推荐使用辅路。
   * @param encParam 屏幕分享的画面编码参数，SDK 会优先使用您通过此接口设置的编码参数：
   *   - 如果您设置 encParam 为 nil，且您已通过 setSubStreamEncoderParam 设置过辅路视频编码参数，SDK
   * 将使用您设置过的辅路编码参数进行屏幕分享。
   *   - 如果您设置 encParam 为 nil，且您未通过 setSubStreamEncoderParam 设置过辅路视频编码参数，SDK
   * 将自动选择一个最佳的编码参数进行屏幕分享。
   * @note
   * 1. 同一个用户同时最多只能发布一路主路（{@link TCCCVideoStreamTypeBig}）画面和一路辅路（{@link
   * TCCCVideoStreamTypeSub}）画面。
   * 2. 默认情况下，屏幕分享使用辅路画面。如果使用主路做屏幕分享，您需要提前停止摄像头采集（{@link
   * stopLocalPreview}）以避免相互冲突。
   * 3. 同一个房间中同时只能有一个用户使用辅路做屏幕分享，也就是说，同一个房间中同时只允许一个用户开启辅路。
   * 4. 当房间中已经有其他用户在使用辅路分享屏幕时，此时调用该接口会收到来自 {@link ITRTCCloudCallback} 的
   * onError(ERR_SERVER_CENTER_ANOTHER_USER_PUSH_SUB_VIDEO) 回调。
   */
  virtual void startScreenCapture(TCCCView view, TCCCVideoStreamType streamType, TCCCVideoEncParam* encParam) = 0;

  /**
   * 9.2 停止屏幕分享
   */
  virtual void stopScreenCapture() = 0;

  /**
   * 9.3 暂停屏幕分享
   */
  virtual void pauseScreenCapture() = 0;

  /**
   * 9.4 恢复屏幕分享
   */
  virtual void resumeScreenCapture() = 0;
  /// @}

  /////////////////////////////////////////////////////////////////////////////////
  //
  //                    设备管理相关接口
  //
  /////////////////////////////////////////////////////////////////////////////////
  /// @name  设备管理相关接口
  /// @{

  /**
   * 10.1 获取设备管理类（ITCCCDeviceManager）
   */
  virtual ITCCCDeviceManager* getDeviceManager() = 0;

  /////////////////////////////////////////////////////////////////////////////////
  //
  //                    调试相关接口
  //
  /////////////////////////////////////////////////////////////////////////////////
  /// @name  调试相关接口
  /// @{
  /**
   * 12.1 获取 SDK 版本信息
   */
  virtual const char* getSDKVersion() = 0;

  /**
   * 12.2 调用实验性接口
   *
   * @param commandType 类型
   * @param jsonStr 参数
   */
  virtual void callExperimentalAPI(const char* commandType, const char* jsonStr) = 0;

  /**
   * 12.3 设置 Log 输出级别，该方法必须在login前调用才生效
   *
   * @param level 参见 {@link TCCCLogLevel}，默认值：{@link TCCCLogLevelInfo}
   */
  virtual void setLogLevel(TCCCLogLevel level) = 0;

  /**
   * 12.4 启用/禁用控制台日志打印
   *
   * @param enabled 指定是否启用，默认:禁止状态
   */
  virtual void setConsoleEnabled(bool enabled) = 0;

  /**
   * 12.5 设置日志文件夹,登录前调用才有效
   *
   * @param logPath 日志文件夹路径
   */
  virtual void setLogDirectory(const char* logPath) = 0;

  /*
   * 当网络异常不能恢复的时候调用，通常用于手机切后台后不能恢复。
   * 不要轻易调用,建议在手机后台切回前台的时候，checkLogin-->onError-->resetSip-->checkLogin
   */
  virtual void resetSip(bool isNeedCallReLogin) = 0;

  /**
   * @cond
   * 只能在内部使用。
   */
  virtual void genTestUserSigByUser(const char* secretId, const char* secretKey, const char* userId,
                                    const char* clientData, uint32_t sdkAppId, ITXCallback* callback) = 0;

  /**
   * @endcond
   */

  /// @}
};
}  // namespace tccc
/// @}

#endif /* TCCC_USER_SDK_TCCC_INCLUDE_ITCCCCLOUD_H_ */
