// Copyright 2020 Tencent Inc. All rights reserved.
//
// Author: Tencent
//
// 文件注释
/**
 * Module:   TCCC 关键类型定义
 * Function: 登录参数、呼叫、会话参数和常量值的定义
 */
/// @defgroup TCCCTypeDef_cplusplus TCCC 关键类型定义
/// 腾讯云联络中心的关键类型定义
/// @{
#ifndef TCCC_USER_SDK_TCCC_INCLUDE_TCCCTypeDef_h
#  define TCCC_USER_SDK_TCCC_INCLUDE_TCCCTypeDef_h

#  include <stdint.h>
#  include <memory>

#  ifdef _WIN32
#    ifndef WIN32_LEAN_AND_MEAN
#      define WIN32_LEAN_AND_MEAN
#    endif
#    include <windows.h>
#    ifdef TCCC_EXPORTS
#      define TCCC_API __declspec(dllexport)
#    else
#      define TCCC_API __declspec(dllimport)
#    endif
#  elif __APPLE__
#    include <TargetConditionals.h>
#    define TCCC_API __attribute__((visibility("default")))
#  elif __ANDROID__
#    define TCCC_API __attribute__((visibility("default")))
#  else
#    define TCCC_API
#  endif

#  define TARGET_PLATFORM_DESKTOP ((__APPLE__ && TARGET_OS_MAC && !TARGET_OS_IPHONE) || _WIN32)
#  define TARGET_PLATFORM_PHONE (__ANDROID__ || (__APPLE__ && TARGET_OS_IOS))
#  define TARGET_PLATFORM_MAC (__APPLE__ && TARGET_OS_MAC && !TARGET_OS_IPHONE)

#  ifdef _WIN32
typedef HWND TCCCView;
#  else
typedef void *TCCCView;
#  endif

namespace tccc {
/////////////////////////////////////////////////////////////////////////////////
//
//                    TCCC 核心类型定义
//
/////////////////////////////////////////////////////////////////////////////////

/**
 * 1.1 登录参数
 */
struct TCCCLoginParams {
  ///【字段含义】应用标识（必填），腾讯云基于 sdkAppId 完成计费统计。暂时无作用可不填写
  ///【推荐取值】在 [联络中心控制台](https://console.cloud.tencent.com/cccc/) 创建应用后可以在账号信息页面中得到该 ID。
  uint32_t sdkAppId;

  ///【字段含义】用户标识（必填），当前用户的 userId，相当于用户名，使用 UTF-8 编码。
  ///【推荐取值】格式 <scheme> : <user> @<host>。如
  /// sip:3013@1400xxxx.tccc.qcloud.com，其中3013为分机号，1400xxxx为你的tccc应用ID
  const char *userId;

  /// 用户数据签名，更多详情请参见[创建用户数据签名](https://cloud.tencent.com/document/product/679/58260)
  const char *userSig;

  /// 用户签名数据
  const char *clientData;

  TCCCLoginParams() : sdkAppId(0), userId(""), userSig(""), clientData("") {}
};
/**
 * 1.2 登录返回值
 */
struct TCCCLoginInfo {
  ///【字段含义】用户标识（必填），当前用户的 userId，相当于用户名，使用 UTF-8 编码。
  ///【推荐取值】如果一个用户在您的账号系统中的 ID 为“mike”，则 userId 即可设置为“mike”。
  const char *userId;
  /// 用户Aor。格式 user@host[:port]
  const char *userAor;
  /// 其他信息，用来定位问题。
  const char *otherInfo;

  TCCCLoginInfo() : userId(nullptr), userAor(nullptr), otherInfo(nullptr) {}
};
/**
 * 1.3 呼叫类型
 */
enum TCCCCallType {
  /// Voip
  Voip = 0,
  /// 视频
  Video = 1,
};

/**
 * 1.4 拨打参数。（内部使用，外部不建议调用）
 */
struct TCCCStartCallParams {
  ///【字段含义】被叫号码（必填），可以是电话号码，使用 UTF-8 编码。格式 <scheme> : <user> @<host>。如
  /// sip:1343xxxx@1400xxxx.tccc.qcloud.com，其中1343xxxx为手机号，1400xxxx为你的tccc应用ID
  const char *to;

  // ///【字段含义】号码备注，在通话条中会替代号码显示（可选）
  // const char *remark;

  // ///【字段含义】户自定义数据（可选），传入后可通过 [电话 CDR
  // ///事件](https://cloud.tencent.com/document/product/679/67257) 推送返回
  // /// 参考 https://cloud.tencent.com/document/product/679/67257
  // const char *uui;

  /// 用户签名数据
  const char *clientData;

  /// 【字段含义】呼叫类型（可选），默认是语音
  TCCCCallType type;

  TCCCStartCallParams() : to(""), clientData(nullptr), type(TCCCCallType::Voip) {}
};

/**
 * 1.4 一键外呼参数
 */
struct TCCCCallParams {
  ///【字段含义】呼叫频道Id
  const char *channelId;

  ///【字段含义】应用标识（必填），腾讯云基于 sdkAppId 完成计费统计。暂时无作用可不填写
  ///【推荐取值】在 [联络中心控制台](https://console.cloud.tencent.com/cccc/) 创建应用后可以在账号信息页面中得到该 ID。
  uint32_t sdkAppId;

  ///【字段含义】用户标识（必填），当前用户的 userId，相当于用户名，使用 UTF-8 编码。
  const char *userId;

  /// 用户数据签名（一次性有效，请每次重新生成），更多详情请参见[创建用户数据签名](https://cloud.tencent.com/document/product/679/58260)
  const char *userSig;

  /// 用户签名数据
  const char *clientData;
  /// 呼叫类型
  TCCCCallType callType;
  TCCCCallParams()
      : channelId(""), sdkAppId(0), userId(""), userSig(""), clientData(""), callType(TCCCCallType::Voip) {}
};

/**
 * 1.5 Tccc 会话信息
 */
enum TCCCSessionDirection {
  /// 呼入
  CallIn = 0,
  /// 呼出
  CallOut = 1,
};

/**
 * 1.5 Tccc 会话方向
 */
struct TCCCSessionInfo {
  ///【字段含义】会话ID。
  const char *sessionID;

  ///【字段含义】被叫（必填），可以是电话号码，使用 UTF-8 编码。
  const char *toUserID;

  // 【字段含义】被叫。格式 <user>@<host>:[:port]
  const char *toAor;

  ///【字段含义】主叫，可以是电话号码，使用 UTF-8 编码。
  const char *fromUserID;

  /// 【字段含义】主叫Aor , 格式 <user>@<host>:[:port]
  const char *fromAor;

  /// 获取所有自定义字段和值
  const char *customHeaderJson;

  /// 获取会话方向
  TCCCSessionDirection sessionDirection;

  TCCCSessionInfo(const char *sessionID_, const char *toUserID_, const char *toAor_, const char *fromUserID_,
                  const char *fromAor_, const char *customHeaderJson_, TCCCSessionDirection sessionDirection_)
      : sessionID(sessionID_),
        toUserID(toUserID_),
        toAor(toAor_),
        fromUserID(fromUserID_),
        fromAor(fromAor_),
        customHeaderJson(customHeaderJson_),
        sessionDirection(sessionDirection_) {}
  TCCCSessionInfo()
      : sessionID(nullptr),
        toUserID(nullptr),
        toAor(nullptr),
        fromUserID(nullptr),
        fromAor(nullptr),
        customHeaderJson(nullptr),
        sessionDirection(TCCCSessionDirection::CallIn) {}
};

/**
 * 1.6 Tccc 转接会话参数
 */
struct TCCCtransferParams {
  const char *sessionId;
  TCCCtransferParams() : sessionId(nullptr) {}
};

/////////////////////////////////////////////////////////////////////////////////
//
//                    网络相关枚举值定义
//
/////////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////////
//
//                    音频相关枚举值定义
//
/////////////////////////////////////////////////////////////////////////////////

/**
 * 3.2 声音音质
 *
 * TCCC 提供了三种精心校调好的模式，用来满足各种垂直场景下对音质的差异化追求：
 * - 人声模式（Speech）：适用于以人声沟通为主的应用场景，该模式下音频传输的抗性较强，TCCC
 * 会通过各种人声处理技术保障在弱网络环境下的流畅度最佳。
 * - 音乐模式（Music）：适用于对声乐要求很苛刻的场景，该模式下音频传输的数据量很大，TCCC
 * 会通过各项技术确保音乐信号在各频段均能获得高保真的细节还原度。
 * - 默认模式（Default）：介于 Speech 和 Music
 * 之间的档位，对音乐的还原度比人声模式要好，但传输数据量比音乐模式要低很多，对各种场景均有不错的适应性。
 */
enum TCCCAudioQuality {

  ///人声模式：采样率：16k；单声道；编码码率：16kbps；具备几个模式中最强的网络抗性，适合语音通话为主的场景，比如在线会议，语音通话等。
  TCCCAudioQualitySpeech = 1,

  ///默认模式：采样率：48k；单声道；编码码率：50kbps；介于 Speech 和 Music 之间的档位，SDK 默认档位，推荐选择。
  TCCCAudioQualityDefault = 2,

  ///音乐模式：采样率：48k；全频带立体声；编码码率：128kbps；适合需要高保真传输音乐的场景，比如在线K歌、音乐直播等。
  TCCCAudioQualityMusic = 3,

};

/**
 * 5.5 网络质量
 *
 * TCCC 会每隔两秒对当前的网络质量进行评估，评估结果为六个等级：Excellent 表示最好，Down 表示最差。
 */
enum TCCCQuality {

  /// 未定义
  TCCCQuality_Unknown = 0,

  /// 当前网络非常好
  TCCCQuality_Excellent = 1,

  /// 当前网络比较好
  TCCCQuality_Good = 2,

  /// 当前网络一般
  TCCCQuality_Poor = 3,

  /// 当前网络较差
  TCCCQuality_Bad = 4,

  /// 当前网络很差
  TCCCQuality_Vbad = 5,

  /// 当前网络不满足 TCCC 的最低要求
  TCCCQuality_Down = 6,

};

/**
 * 5.6 网络质量
 *
 * 表征网络质量的好坏，您可以通过该数值在用户界面上展示每个用户的网络质量。
 */
struct TCCCQualityInfo {
  /// 用户 ID
  const char *userId;

  /// 网络质量
  TCCCQuality quality;
  TCCCQualityInfo() : userId(nullptr), quality(TCCCQuality_Unknown) {}
};

/**
 * 5.7 测速参数
 *
 * 您可以在用户进入房间前通过 {@link startSpeedTest} 接口测试网速（注意：请不要在通话中调用）。
 */
struct TCCCSpeedTestParams {
  /// 应用标识，请参考 {@link TCCCSpeedTestParams} 中的相关说明。
  int sdkAppId;

  /// 用户标识，请参考 {@link TCCCSpeedTestParams} 中的相关说明。
  const char *userId;

  /// 用户签名，请参考 {@link TCCCSpeedTestParams} 中的相关说明。
  const char *userSig;

  ///预期的上行带宽（kbps，取值范围： 10 ～ 5000，为 0 时不测试）。
  int expectedUpBandwidth;

  /// 预期的下行带宽（kbps，取值范围： 10 ～ 5000，为 0 时不测试）。
  int expectedDownBandwidth;

  TCCCSpeedTestParams()
      : sdkAppId(0), userId(nullptr), userSig(nullptr), expectedUpBandwidth(0), expectedDownBandwidth(0) {}
};

/**
 * 5.8 网络测速结果
 *
 * 您可以在用户进入房间前通过 {@link startSpeedTest:} 接口进行测速（注意：请不要在通话中调用）。
 */
struct TCCCSpeedTestResult {
  /// 测试是否成功。
  bool success;

  /// 带宽测试错误信息。
  const char *errMsg;

  ///服务器 IP 地址。
  const char *ip;

  /// 内部通过评估算法测算出的网络质量，更多信息请参见 {@link TCCCQuality}。
  TCCCQuality quality;

  /// 上行丢包率，取值范围是 [0 - 1.0]，例如 0.3 表示每向服务器发送 10 个数据包可能会在中途丢失 3 个。
  float upLostRate;

  /// 下行丢包率，取值范围是 [0 - 1.0]，例如 0.2 表示每从服务器收取 10 个数据包可能会在中途丢失 2 个。
  float downLostRate;

  /// 延迟（毫秒），指当前设备到 RTC 服务器的一次网络往返时间，该值越小越好，正常数值范围是10ms - 100ms。
  int rtt;

  /// 上行带宽（kbps，-1：无效值）。
  int availableUpBandwidth;

  /// 下行带宽（kbps，-1：无效值）。
  int availableDownBandwidth;

  TCCCSpeedTestResult()
      : success(false),
        errMsg(nullptr),
        ip(nullptr),
        quality(TCCCQuality_Unknown),
        upLostRate(0.0f),
        downLostRate(0.0f),
        rtt(0),
        availableUpBandwidth(0),
        availableDownBandwidth(0) {}
};

/**
 * 5.9 音量大小信息
 */
struct TCCCVolumeInfo {
  /// 说话者的 userId, 如果 userId 为空则代表是当前用户自己。
  const char *userId;
  /// 说话者的音量大小, 取值范围[0 - 100]。
  uint32_t volume;
  TCCCVolumeInfo() : userId(""), volume(0) {}
};

/////////////////////////////////////////////////////////////////////////////////
//
//                      更多枚举值定义
//
/////////////////////////////////////////////////////////////////////////////////

/**
 * 4.1 Log 级别
 *
 * 不同的日志等级定义了不同的详实程度和日志数量，推荐一般情况下将日志等级设置为：TCCCLogLevelInfo。
 */
enum TCCCLogLevel {
  /// 输出所有级别的 Log
  TCCCLogLevelVerbose = 0,

  /// 输出 DEBUG，INFO，WARNING，ERROR 和 FATAL 级别的 Log
  TCCCLogLevelDebug = 1,

  /// 输出 INFO，WARNING，ERROR 和 FATAL 级别的 Log
  TCCCLogLevelInfo = 2,

  /// 输出WARNING，ERROR 和 FATAL 级别的 Log
  TCCCLogLevelWarn = 3,

  /// 输出ERROR 和 FATAL 级别的 Log
  TCCCLogLevelError = 4,

  /// 仅输出 FATAL 级别的 Log
  TCCCLogLevelFatal = 5,

  ///不输出任何 SDK Log
  TCCCLogLevelNone = 6,

};

/**
 * 5.1 视频流类型
 *
 * TCCC 内部有三种不同的视频流，分别是：
 *  - 高清大画面：一般用来传输摄像头的视频数据。
 *  - 低清小画面：小画面和大画面的内容相互，但是分辨率和码率都比大画面低，因此清晰度也更低。
 *  -
 * 辅流画面：一般用于屏幕分享，同一时间在同一个房间中只允许一个用户发布辅流视频，其他用户必须要等该用户关闭之后才能发布自己的辅流。
 * @note 不支持单独开启低清小画面，小画面必须依附于大画面而存在，SDK 会自动设定低清小画面的分辨率和码率。
 */
enum TCCCVideoStreamType {

  /// 高清大画面，一般用来传输摄像头的视频数据。
  TCCCVideoStreamTypeBig = 0,

  /// 低清小画面：小画面和大画面的内容相互，但是分辨率和码率都比大画面低，因此清晰度也更低。
  TCCCVideoStreamTypeSmall = 1,

  /// 辅流画面：一般用于屏幕分享，同一时间在同一个房间中只允许一个用户发布辅流视频，其他用户必须要等该用户关闭之后才能发布自己的辅流。
  TCCCVideoStreamTypeSub = 2,

};

/**
 * 5.2 视频分辨率
 *
 * 此处仅定义横屏分辨率（如 640 × 360），如需使用竖屏分辨率（如360 × 640），需要同时指定 TCCCVideoResolutionMode 为
 * Portrait。
 */
enum TCCCVideoResolution {

  /// 宽高比 1:1；分辨率 120x120；建议码率（VideoCall）80kbps; 建议码率（LIVE）120kbps。
  TCCCVideoResolution_120_120 = 1,

  /// 宽高比 1:1 分辨率 160x160；建议码率（VideoCall）100kbps; 建议码率（LIVE）150kbps。
  TCCCVideoResolution_160_160 = 3,

  /// 宽高比 1:1；分辨率 270x270；建议码率（VideoCall）200kbps; 建议码率（LIVE）300kbps。
  TCCCVideoResolution_270_270 = 5,

  /// 宽高比 1:1；分辨率 480x480；建议码率（VideoCall）350kbps; 建议码率（LIVE）500kbps。
  TCCCVideoResolution_480_480 = 7,

  /// 宽高比4:3；分辨率 160x120；建议码率（VideoCall）100kbps; 建议码率（LIVE）150kbps。
  TCCCVideoResolution_160_120 = 50,

  /// 宽高比 4:3；分辨率 240x180；建议码率（VideoCall）150kbps; 建议码率（LIVE）250kbps。
  TCCCVideoResolution_240_180 = 52,

  /// 宽高比 4:3；分辨率 280x210；建议码率（VideoCall）200kbps; 建议码率（LIVE）300kbps。
  TCCCVideoResolution_280_210 = 54,

  /// 宽高比 4:3；分辨率 320x240；建议码率（VideoCall）250kbps; 建议码率（LIVE）375kbps。
  TCCCVideoResolution_320_240 = 56,

  /// 宽高比 4:3；分辨率 400x300；建议码率（VideoCall）300kbps; 建议码率（LIVE）450kbps。
  TCCCVideoResolution_400_300 = 58,

  /// 宽高比 4:3；分辨率 480x360；建议码率（VideoCall）400kbps; 建议码率（LIVE）600kbps。
  TCCCVideoResolution_480_360 = 60,

  /// 宽高比 4:3；分辨率 640x480；建议码率（VideoCall）600kbps; 建议码率（LIVE）900kbps。
  TCCCVideoResolution_640_480 = 62,

  /// 宽高比 4:3；分辨率 960x720；建议码率（VideoCall）1000kbps; 建议码率（LIVE）1500kbps。
  TCCCVideoResolution_960_720 = 64,

  /// 宽高比 16:9；分辨率 160x90；建议码率（VideoCall）150kbps; 建议码率（LIVE）250kbps。
  TCCCVideoResolution_160_90 = 100,

  /// 宽高比 16:9；分辨率 256x144；建议码率（VideoCall）200kbps; 建议码率（LIVE）300kbps。
  TCCCVideoResolution_256_144 = 102,

  /// 宽高比 16:9；分辨率 320x180；建议码率（VideoCall）250kbps; 建议码率（LIVE）400kbps。
  TCCCVideoResolution_320_180 = 104,

  /// 宽高比 16:9；分辨率 480x270；建议码率（VideoCall）350kbps; 建议码率（LIVE）550kbps。
  TCCCVideoResolution_480_270 = 106,

  /// 宽高比 16:9；分辨率 640x360；建议码率（VideoCall）500kbps; 建议码率（LIVE）900kbps。
  TCCCVideoResolution_640_360 = 108,

  /// 宽高比 16:9；分辨率 960x540；建议码率（VideoCall）850kbps; 建议码率（LIVE）1300kbps。
  TCCCVideoResolution_960_540 = 110,

  /// 宽高比 16:9；分辨率 1280x720；建议码率（VideoCall）1200kbps; 建议码率（LIVE）1800kbps。
  TCCCVideoResolution_1280_720 = 112,

  /// 宽高比 16:9；分辨率 1920x1080；建议码率（VideoCall）2000kbps; 建议码率（LIVE）3000kbps。
  TCCCVideoResolution_1920_1080 = 114,

};

/**
 * 5.3 视频宽高比模式
 *
 * TCCCVideoResolution 中仅定义了横屏分辨率（如 640 × 360），如需使用竖屏分辨率（如360 × 640），需要同时指定
 * TCCCVideoResolutionMode 为 Portrait。
 */
enum TCCCVideoResolutionMode {

  /// 横屏分辨率，例如：TCCCVideoResolution_640_360 + TCCCVideoResolutionModeLandscape = 640 × 360。
  TCCCVideoResolutionModeLandscape = 0,

  /// 竖屏分辨率，例如：TCCCVideoResolution_640_360 + TCCCVideoResolutionModePortrait  = 360 × 640。
  TCCCVideoResolutionModePortrait = 1,

};

/**
 * 5.4 视频画面填充模式
 *
 * 如果视频显示区域的宽高比不等于视频内容的宽高比时，需要指定画面的填充模式。
 */
enum TCCCVideoFillMode {

  /// 填充模式：即将画面内容居中等比缩放以充满整个显示区域，超出显示区域的部分将会被裁剪掉，此模式下画面可能不完整。
  TCCCVideoFillMode_Fill = 0,

  /// 适应模式：即按画面长边进行缩放以适应显示区域，短边部分会被填充为黑色，此模式下图像完整但可能留有黑边。
  TCCCVideoFillMode_Fit = 1,

};

/**
 * 5.5 视频画面旋转方向
 *
 * TCCC 提供了对本地和远程画面的旋转角度设置 API，下列的旋转角度都是指顺时针方向的。
 */
enum TCCCVideoRotation {

  /// 不旋转。
  TCCCVideoRotation0 = 0,

  /// 顺时针旋转90度。
  TCCCVideoRotation90 = 1,

  /// 顺时针旋转180度。
  TCCCVideoRotation180 = 2,

  /// 顺时针旋转270度。
  TCCCVideoRotation270 = 3,

};

/**
 * 5.9 视频的镜像类型
 *
 * 视频的镜像是指对视频内容进行左右翻转，尤其是对本地的摄像头预览视频，开启镜像后能给主播带来熟悉的“照镜子”体验。
 */
enum TCCCVideoMirrorType {

/// 自动模式：如果正使用前置摄像头则开启镜像，如果是后置摄像头则不开启镜像（仅适用于移动设备）。
#  if TARGET_PLATFORM_PHONE
  TCCCVideoMirrorType_Auto = 0,
#  endif

  /// 强制开启镜像，不论当前使用的是前置摄像头还是后置摄像头。
  TCCCVideoMirrorType_Enable = 1,

  /// 强制关闭镜像，不论当前使用的是前置摄像头还是后置摄像头。
  TCCCVideoMirrorType_Disable = 2,

};

/**
 * 5.2 视频编码参数
 *
 * 该设置决定远端用户看到的画面质量，同时也决定了云端录制出的视频文件的画面质量。
 */
struct TCCCVideoEncParam {
  ///【字段含义】视频分辨率。
  ///【特别说明】如需使用竖屏分辨率，请指定 resMode 为 Portrait，例如： 640 × 360 + Portrait = 360 × 640。
  ///【推荐取值】
  /// - 手机视频通话：建议选择 360 × 640 及以下分辨率，resMode 选择 Portrait，即竖屏分辨率。
  /// - 手机在线直播：建议选择 540 × 960，resMode 选择 Portrait，即竖屏分辨率。
  /// - 桌面平台（Win + Mac）：建议选择 640 × 360 及以上分辨率，resMode 选择 Landscape，即横屏分辨率。
  TCCCVideoResolution videoResolution;

  ///【字段含义】分辨率模式（横屏分辨率 or 竖屏分辨率）。
  ///【推荐取值】手机平台（iOS、Android）建议选择 Portrait，桌面平台（Windows、Mac）建议选择 Landscape。
  ///【特别说明】如需使用竖屏分辨率，请指定 resMode 为 Portrait，例如： 640 × 360 + Portrait = 360 × 640。
  TCCCVideoResolutionMode resMode;

  ///【字段含义】视频采集帧率。
  ///【推荐取值】15fps或20fps。5fps以下，卡顿感明显。10fps以下，会有轻微卡顿感。20fps以上，会浪费带宽（电影的帧率为24fps）。
  ///【特别说明】部分 Android 手机的前置摄像头并不支持15fps以上的采集帧率，部分主打美颜功能的 Android
  /// 手机的前置摄像头的采集帧率可能低于10fps。
  uint32_t videoFps;

  ///【字段含义】目标视频码率，SDK 会按照目标码率进行编码，只有在弱网络环境下才会主动降低视频码率。
  ///【推荐取值】请参见本 TCCCVideoResolution
  /// 在各档位注释的最佳码率，也可以在此基础上适当调高。比如：TCCCVideoResolution_1280_720 对应 1200kbps
  /// 的目标码率，您也可以设置为 1500kbps 用来获得更好的观感清晰度。 【特别说明】您可以通过同时设置 videoBitrate 和
  /// minVideoBitrate 两个参数，用于约束 SDK 对视频码率的调整范围：
  /// - 如果您追求“弱网络下允许卡顿但要保持清晰”的效果，可以设置 minVideoBitrate 为 videoBitrate 的 60%。
  /// - 如果您追求“弱网络下允许模糊但要保持流畅”的效果，可以设置 minVideoBitrate 为一个较低的数值（比如 100kbps）。
  /// - 如果您将 videoBitrate 和 minVideoBitrate 设置为同一个值，等价于关闭 SDK 对视频码率的自适应调节能力。
  uint32_t videoBitrate;

  ///【字段含义】最低视频码率，SDK 会在网络不佳的情况下主动降低视频码率以保持流畅度，最低会降至 minVideoBitrate
  /// 所设定的数值。 【特别说明】 默认值：0，此时最低码率由 SDK 会根据您指定的分辨率，自动计算出合适的数值。
  ///【推荐取值】您可以通过同时设置 videoBitrate 和 minVideoBitrate 两个参数，用于约束 SDK 对视频码率的调整范围：
  /// - 如果您追求`弱网络下允许卡顿但要保持清晰`的效果，可以设置 minVideoBitrate 为 videoBitrate 的 60%。
  /// - 如果您追求`弱网络下允许模糊但要保持流畅`的效果，可以设置 minVideoBitrate 为一个较低的数值（比如 100kbps）。
  /// - 如果您将 videoBitrate 和 minVideoBitrate 设置为同一个值，等价于关闭 SDK 对视频码率的自适应调节能力。
  uint32_t minVideoBitrate;

  ///【字段含义】是否允许动态调整分辨率（开启后会对云端录制产生影响）。
  ///【推荐取值】该功能适用于不需要云端录制的场景，开启后 SDK
  /// 会根据当前网络情况，智能选择出一个合适的分辨率，避免出现“大分辨率+小码率”的低效编码模式。
  ///【特别说明】默认值：关闭。如有云端录制的需求，请不要开启此功能，因为如果视频分辨率发生变化后，云端录制出的 MP4
  /// 在普通的播放器上无法正常播放。
  bool enableAdjustRes;

  TCCCVideoEncParam()
      : videoResolution(TCCCVideoResolution_640_360),
        resMode(TCCCVideoResolutionModeLandscape),
        videoFps(15),
        videoBitrate(550),
        minVideoBitrate(0),
        enableAdjustRes(false) {}
};

/**
 * 5.4 视频画面的渲染参数
 *
 * 您可以通过设置此参数来控制画面的旋转角度、填充模式和左右镜像模式。
 */
struct TCCCRenderParams {
  ///【字段含义】图像的顺时针旋转角度。
  ///【推荐取值】支持 90、180 以及 270 旋转角度，默认值：{@link TCCCVideoRotation0}。
  TCCCVideoRotation rotation;

  ///【字段含义】画面填充模式。
  ///【推荐取值】填充（画面可能会被拉伸裁剪）或适应（画面可能会有黑边），默认值：{@link TCCCVideoFillMode_Fill}。
  TCCCVideoFillMode fillMode;

  ///【字段含义】画面镜像模式。
  ///【推荐取值】默认值：{@link TCCCVideoMirrorType_Auto}。
  TCCCVideoMirrorType mirrorType;

  TCCCRenderParams()
      : rotation(TCCCVideoRotation0), fillMode(TCCCVideoFillMode_Fit), mirrorType(TCCCVideoMirrorType_Disable) {}
};

/////////////////////////////////////////////////////////////////////////////////
//
//                    网络和性能的汇总统计指标
//
/////////////////////////////////////////////////////////////////////////////////

/**
 * 本地的音视频统计指标
 */
struct TCCCLocalStatistics {
  ///【字段含义】本地视频的宽度，单位 px。
  uint32_t width;

  ///【字段含义】本地视频的高度，单位 px。
  uint32_t height;

  ///【字段含义】本地视频的帧率，即每秒钟会有多少视频帧，单位：FPS。
  uint32_t frameRate;

  ///【字段含义】本地视频的码率，即每秒钟新产生视频数据的多少，单位 Kbps。
  uint32_t videoBitrate;

  ///【字段含义】本地音频的采样率，单位 Hz。
  uint32_t audioSampleRate;

  ///【字段含义】本地音频的码率，即每秒钟新产生音频数据的多少，单位 Kbps。
  uint32_t audioBitrate;

  ///【字段含义】视频流类型（高清大画面|低清小画面|辅流画面）。
  TCCCVideoStreamType streamType;

  ///【字段含义】音频设备采集状态（用于检测音频外设的健康度）。
  /// 0：采集设备状态正常；1：检测到长时间静音；2：检测到破音；3：检测到声音异常间断。
  uint32_t audioCaptureState;

  TCCCLocalStatistics()
      : width(0),
        height(0),
        frameRate(0),
        videoBitrate(0),
        audioSampleRate(0),
        audioBitrate(0),
        streamType(TCCCVideoStreamTypeBig),
        audioCaptureState(0) {}
};

/**
 * 远端的音视频统计指标
 */
struct TCCCRemoteStatistics {
  ///【字段含义】用户 ID
  char *userId;

  ///【字段含义】音频流的总丢包率（％）。
  /// audioPacketLoss 代表音频流历经`主播>云端>观众`这样一条完整的传输链路后，最终在观众端统计到的丢包率。
  /// audioPacketLoss 越小越好，丢包率为0即表示该路音频流的所有数据均已经完整地到达了观众端。
  /// 如果出现了 downLoss == 0 但 audioPacketLoss != 0
  /// 的情况，说明该路音频流在“云端=>观众”这一段链路上没有出现丢包，但是在`主播>云端`这一段链路上出现了不可恢复的丢包。
  uint32_t audioPacketLoss;

  ///【字段含义】该路视频流的总丢包率（％）。
  /// videoPacketLoss 代表该路视频流历经`主播>云端>观众`这样一条完整的传输链路后，最终在观众端统计到的丢包率。
  /// videoPacketLoss 越小越好，丢包率为0即表示该路视频流的所有数据均已经完整地到达了观众端。
  /// 如果出现了 downLoss == 0 但 videoPacketLoss != 0
  /// 的情况，说明该路视频流在`云端>观众`这一段链路上没有出现丢包，但是在`主播>云端`这一段链路上出现了不可恢复的丢包。
  uint32_t videoPacketLoss;

  ///【字段含义】远端视频的宽度，单位 px。
  uint32_t width;

  ///【字段含义】远端视频的高度，单位 px。
  uint32_t height;

  ///【字段含义】远端视频的帧率，单位：FPS。
  uint32_t frameRate;

  ///【字段含义】远端视频的码率，单位 Kbps。
  uint32_t videoBitrate;

  ///【字段含义】本地音频的采样率，单位 Hz。
  uint32_t audioSampleRate;

  ///【字段含义】本地音频的码率，单位 Kbps
  uint32_t audioBitrate;

  ///【字段含义】播放延迟，单位 ms
  /// 为了避免网络抖动和网络包乱序导致的声音和画面卡顿，TCCC
  /// 会在播放端管理一个播放缓冲区，用于对接收到的网络数据包进行整理，该缓冲区的大小会根据当前的网络质量进行自适应调整，该缓冲区的大小折算成以毫秒为单位的时间长度，也就是
  /// jitterBufferDelay。
  uint32_t jitterBufferDelay;

  ///【字段含义】端到端延迟，单位 ms
  /// point2PointDelay 代表 “主播=>云端=>观众”
  /// 的延迟，更准确地说，它代表了“采集=>编码=>网络传输=>接收=>缓冲=>解码=>播放” 全链路的延迟。 point2PointDelay
  /// 需要本地和远端的 SDK 均为 8.5 及以上的版本才生效，若远端用户为 8.5 以前的版本，此数值会一直为0，代表无意义。
  uint32_t point2PointDelay;

  ///【字段含义】音频播放的累计卡顿时长，单位 ms
  uint32_t audioTotalBlockTime;

  ///【字段含义】音频播放卡顿率，单位 (%)
  /// 音频播放卡顿率（audioBlockRate） = 音频播放的累计卡顿时长（audioTotalBlockTime） / 音频播放的总时长
  uint32_t audioBlockRate;

  ///【字段含义】视频播放的累计卡顿时长，单位 ms
  uint32_t videoTotalBlockTime;

  ///【字段含义】视频播放卡顿率，单位 (%) 视频播放卡顿率（videoBlockRate） =
  ///视频播放的累计卡顿时长（videoTotalBlockTime） / 视频播放的总时长。
  uint32_t videoBlockRate;

  ///【字段含义】该路音视频流的总丢包率（％）。已废弃，不推荐使用；建议使用 audioPacketLoss、videoPacketLoss 替代。
  uint32_t finalLoss;

  ///【字段含义】视频流类型（高清大画面|低清小画面|辅流画面）。
  TCCCVideoStreamType streamType;

  TCCCRemoteStatistics()
      : userId(nullptr),
        audioPacketLoss(0),
        videoPacketLoss(0),
        width(0),
        height(0),
        frameRate(0),
        videoBitrate(0),
        audioSampleRate(0),
        audioBitrate(0),
        jitterBufferDelay(0),
        point2PointDelay(0),
        audioTotalBlockTime(0),
        audioBlockRate(0),
        videoTotalBlockTime(0),
        videoBlockRate(0),
        finalLoss(0),
        streamType(TCCCVideoStreamTypeBig) {}

  TCCCRemoteStatistics(const TCCCRemoteStatistics &temp) {
    int nLength = strlen(temp.userId);
    userId = new char[nLength + 1];
    memset(userId, 0, sizeof(*userId));
    strcpy(userId, temp.userId);
    audioPacketLoss = temp.audioBitrate;
    videoPacketLoss = temp.videoPacketLoss;
    width = temp.width;
    height = temp.height;
    frameRate = temp.frameRate;
    videoBitrate = temp.videoBitrate;
    audioSampleRate = temp.audioSampleRate;
    audioBitrate = temp.audioBitrate;
    jitterBufferDelay = temp.jitterBufferDelay;
    point2PointDelay = temp.point2PointDelay;
    audioTotalBlockTime = temp.audioTotalBlockTime;
    audioBlockRate = temp.audioBlockRate;
    videoTotalBlockTime = temp.videoTotalBlockTime;
    videoBlockRate = temp.videoBlockRate;
    finalLoss = temp.finalLoss;
    streamType = temp.streamType;
  }
  ~TCCCRemoteStatistics() { delete[] userId; }
};

/**
 * 网络和性能的汇总统计指标
 */
struct TCCCStatistics {
  ///【字段含义】当前应用的 CPU 使用率，单位 (%)，Android 8.0 以上不支持。
  uint32_t appCpu;

  ///【字段含义】当前系统的 CPU 使用率，单位 (%)，Android 8.0 以上不支持。
  uint32_t systemCpu;

  ///【字段含义】从 SDK 到云端的上行丢包率，单位 (%)
  ///该数值越小越好，如果 upLoss 为 0%，则意味着上行链路的网络质量很好，上传到云端的数据包基本不发生丢失。
  ///如果 upLoss 为 30%，则意味着 SDK 向云端发送的音视频数据包中，会有 30% 丢失在传输链路中。
  uint32_t upLoss;

  ///【字段含义】从云端到 SDK 的下行丢包率，单位 (%)
  /// 该数值越小越好，如果 downLoss 为 0%，则意味着下行链路的网络质量很好，从云端接收的数据包基本不发生丢失。
  /// 如果 downLoss 为 30%，则意味着云端向 SDK 传输的音视频数据包中，会有 30% 丢失在传输链路中。
  uint32_t downLoss;

  ///【字段含义】从 SDK 到云端的往返延时，单位 ms
  /// 该数值代表从 SDK 发送一个网络包到云端，再从云端回送一个网络包到 SDK 的总计耗时，也就是一个网络包经历
  ///“SDK=>云端=>SDK” 的总耗时。 该数值越小越好：如果 rtt < 50ms，意味着较低的音视频通话延迟；如果 rtt >
  /// 200ms，则意味着较高的音视频通话延迟。 需要特别解释的是，rtt 代表 “SDK=>云端=>SDK” 的总耗时，所不需要区分 upRtt 和
  /// downRtt。
  uint32_t rtt;

  ///【字段含义】从 SDK 到本地路由器的往返时延，单位 ms 该数值代表从 SDK
  /// 发送一个网络包到本地路由器网关，再从网关回送一个网络包到 SDK 的总计耗时，也就是一个网络包经历 【SDK>网关>SDK【
  /// 的总耗时。 该数值越小越好：如果 gatewayRtt < 50ms，意味着较低的音视频通话延迟；如果 gatewayRtt >
  /// 200ms，则意味着较高的音视频通话延迟。 当网络类型为蜂窝网时，该值无效。
  uint32_t gatewayRtt;

  ///【字段含义】总发送字节数（包含信令数据和音视频数据），单位：字节数（Bytes）。
  uint32_t sentBytes;

  ///【字段含义】总接收字节数（包含信令数据和音视频数据），单位：字节数（Bytes）。
  uint32_t receivedBytes;

  ///【字段含义】本地的音视频统计信息
  /// 由于本地可能有三路音视频流（即高清大画面，低清小画面，以及辅流画面），因此本地的音视频统计信息是一个数组。
  TCCCLocalStatistics *localStatisticsArray;

  ///【字段含义】数组 localStatisticsArray 的大小
  uint32_t localStatisticsArraySize;

  ///【字段含义】远端的音视频统计信息
  /// 因为同时可能有多个远端用户，而且每个远端用户同时可能有多路音视频流（即高清大画面，低清小画面，以及辅流画面），因此远端的音视频统计信息是一个数组。
  TCCCRemoteStatistics *remoteStatisticsArray;

  ///【字段含义】数组 remoteStatisticsArray 的大小。
  uint32_t remoteStatisticsArraySize;

  TCCCStatistics()
      : appCpu(0),
        systemCpu(0),
        upLoss(0),
        downLoss(0),
        rtt(0),
        gatewayRtt(0),
        sentBytes(0),
        receivedBytes(0),
        localStatisticsArray(nullptr),
        localStatisticsArraySize(0),
        remoteStatisticsArray(nullptr),
        remoteStatisticsArraySize(0) {}
};

}  // namespace tccc

#endif /* TCCC_USER_SDK_TCCC_INCLUDE_TCCCTypeDef_h */
       /// @}
